'use strict'

const fetch = require('node-fetch')
const { clean } = require('semver')

const nodeDist = 'https://nodejs.org/dist/index.json'
const npmRegistry = 'https://registry.npmjs.com/npm/'

const parseJson = res => res.ok ? res.json() : Promise.reject(new Error(res.statusText))
const fetchJson = url => fetch(url).then(parseJson)

/* NODE */

const nodeLatest = () =>  fetchJson(nodeDist)
    .then(dists => clean(dists[0].version))

const nodeLts = () => fetchJson(nodeDist)
    .then(dists => clean(dists.find(dist => !!dist.lts).version))

const nodeCurrent = () => clean(process.version)

/* NPM */

const npmLatest = () => fetchJson(npmRegistry + 'latest')
    .then(npm => clean(npm.version))

const npmLts = () => fetchJson(npmRegistry + 'lts')
    .then(npm => clean(npm.version))

const npmNext = () => fetchJson(npmRegistry + 'next')
    .then(npm => clean(npm.version))

const npmCurrent = () => {
    try {
        const npm = require('requireg')('npm')
        return clean(npm.version)
    } catch (err) {
        return NaN
    }
}

/* EXPORTS */

module.exports.node = {
    latest: nodeLatest,
    lts: nodeLts,
    current: nodeCurrent
}

module.exports.npm = {
    latest: npmLatest,
    lts: npmLts,
    next: npmNext,
    current: npmCurrent
}
